package com.test.test.mapper;

import com.test.test.domain.PerformanceTest;

import java.util.List;

/**
 * 性能测试Mapper接口
 * 
 * @author test
 * @date 2025-04-14
 */
public interface PerformanceTestMapper 
{
    /**
     * 查询性能测试
     * 
     * @param id 性能测试主键
     * @return 性能测试
     */
    public PerformanceTest selectPerformanceTestById(Long id);

    /**
     * 查询性能测试列表
     * 
     * @param performanceTest 性能测试
     * @return 性能测试集合
     */
    public List<PerformanceTest> selectPerformanceTestList(PerformanceTest performanceTest);

    /**
     * 新增性能测试
     * 
     * @param performanceTest 性能测试
     * @return 结果
     */
    public int insertPerformanceTest(PerformanceTest performanceTest);

    /**
     * 修改性能测试
     * 
     * @param performanceTest 性能测试
     * @return 结果
     */
    public int updatePerformanceTest(PerformanceTest performanceTest);

    /**
     * 删除性能测试
     * 
     * @param id 性能测试主键
     * @return 结果
     */
    public int deletePerformanceTestById(Long id);

    /**
     * 批量删除性能测试
     * 
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deletePerformanceTestByIds(Long[] ids);
}
